@@ -3,7 +3,7 @@ from typing import Iterable, List, Optional
 from quickmpc import JobStatus
 
 
-def make_statuses_detailed(statuses: Optional[Iterable[int]]) \
+def make_statuses_detailed(statuses: Optional[Iterable[JobStatus.ValueType]]) \
         -> Optional[List[str]]:
     if statuses is None:
         return None
