@@ -2,7 +2,7 @@
 
 import logging
 
-from quickmpc import QMPC
+from quickmpc import QMPC, Schema, ShareValueTypeEnum
 
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG)
@@ -18,7 +18,11 @@ if __name__ == '__main__':
     # 複数のシェアを作成・送信
     data_ids = []
     for i in range(3):
-        res = qmpc.send_share([[1]], ["key"])
+        res = qmpc.send_share(
+            [[1]],
+            [Schema(name="key",
+                    type=ShareValueTypeEnum.SHARE_VALUE_TYPE_FIXED_POINT)]
+        )
         data_ids.append(res['data_id'])
         logger.info(res)
 
