@@ -8,16 +8,18 @@ import struct
 import time
 from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass, field, InitVar
-from typing import Any, Dict, Iterable, List, Optional, Tuple
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 from urllib.parse import urlparse
 
+import google.protobuf.json_format
 import grpc
 import numpy as np
 import tqdm  # type: ignore
 from grpc_status import rpc_status  # type: ignore
 
-from .exception import ArgmentError, QMPCJobError, QMPCServerError
-from .proto.common_types.common_types_pb2 import JobErrorInfo, JobStatus
+from .exception import ArgumentError, QMPCJobError, QMPCServerError
+from .proto.common_types.common_types_pb2 import (JobErrorInfo, JobStatus,
+                                                  Schema, ShareValueTypeEnum)
 from .proto.libc_to_manage_pb2 import (DeleteSharesRequest,
                                        ExecuteComputationRequest,
                                        GetComputationResultRequest,
@@ -30,7 +32,7 @@ from .proto.libc_to_manage_pb2_grpc import LibcToManageStub
 from .share import Share
 from .utils.if_present import if_present
 from .utils.make_pieces import MakePiece
-from .utils.overload_tools import Dim2, Dim3, methoddispatch
+from .utils.overload_tools import Dim1, Dim2, Dim3, methoddispatch
 from .utils.parse_csv import format_check
 
 abs_file = os.path.abspath(__file__)
@@ -45,6 +47,8 @@ class QMPCServer:
     __client_stubs: Tuple[LibcToManageStub] = field(init=False)
     __party_size: int = field(init=False)
     token: str
+    retry_num: int = 10
+    retry_wait_time: int = 5
 
     def __post_init__(self, endpoints: List[str]) -> None:
         stubs = [LibcToManageStub(QMPCServer.__create_grpc_channel(ep))
@@ -66,14 +70,14 @@ class QMPCServer:
             channel = grpc.secure_channel(o.netloc, credential)
         else:
             logger.error(f'仕様を満たさない形式のendpointが渡された: {endpoint}')
-            raise ArgmentError(
+            raise ArgumentError(
                 "endpointsにサポートされてないプロトコルが指定されています．http/httpsのいずれかを指定してください．")
 
         return channel
 
     @staticmethod
     def _argument_check(join_order: Tuple[List, List, List]):
-        if len(join_order[0])-1 != len(join_order[1]):
+        if len(join_order[0]) - 1 != len(join_order[1]):
             logger.error(
                 'the size of join must be one less than the size of dataIds')
             return False
@@ -86,6 +90,37 @@ class QMPCServer:
             return False
         return True
 
+    def __retry(self, f: Callable, *request: Any) -> Any:
+        for _ in range(self.retry_num):
+            try:
+                return f(*request)
+            except grpc.RpcError as e:
+                logger.error(f'{e.details()} ({e.code()})')
+
+                # エラーが詳細な情報を持っているか確認
+                status = rpc_status.from_call(e)
+                if status is not None:
+                    for detail in status.details:
+                        if detail.Is(
+                            JobErrorInfo.DESCRIPTOR
+                        ):
+                            # CC で Job 実行時にエラーが発生していた場合
+                            # 例外を rethrow する
+                            err_info = JobErrorInfo()
+                            detail.Unpack(err_info)
+                            logger.error(f"job error information: {err_info}")
+
+                            raise QMPCJobError(err_info) from e
+
+                # MC で Internal Server Error が発生している場合
+                # 例外を rethrow する
+                if e.code() == grpc.StatusCode.UNKNOWN:
+                    raise QMPCServerError("backend server return error") from e
+            except Exception as e:
+                logger.error(e)
+            time.sleep(self.retry_wait_time)
+        raise RuntimeError(f"All {self.retry_num} times it was an error")
+
     @staticmethod
     def __futures_result(
             futures: Iterable, enable_progress_bar=True) -> Tuple[bool, List]:
@@ -96,29 +131,8 @@ class QMPCServer:
             if enable_progress_bar:
                 futures = tqdm.tqdm(futures, desc='receive')
             response = [f.result() for f in futures]
-        except grpc.RpcError as e:
-            is_ok = False
-            logger.error(f'{e.details()} ({e.code()})')
-
-            # エラーが詳細な情報を持っているか確認
-            status = rpc_status.from_call(e)
-            if status is not None:
-                for detail in status.details:
-                    if detail.Is(
-                        JobErrorInfo.DESCRIPTOR   # type: ignore[attr-defined]
-                    ):
-                        # CC で Job 実行時にエラーが発生していた場合
-                        # 例外を rethrow する
-                        err_info = JobErrorInfo()
-                        detail.Unpack(err_info)
-                        logger.error(f"job error information: {err_info}")
-
-                        raise QMPCJobError(err_info) from e
-
-            # MC で Internal Server Error が発生している場合
-            # 例外を rethrow する
-            if e.code() == grpc.StatusCode.UNKNOWN:
-                raise QMPCServerError("backend server return error") from e
+        except (QMPCJobError, QMPCServerError):
+            raise
         except Exception as e:
             is_ok = False
             logger.error(e)
@@ -165,13 +179,38 @@ class QMPCServer:
         res_dict: Dict = {"is_ok": is_ok, "responses": res_list}
         return res_dict
 
+    @methoddispatch(is_static_method=True)
+    @staticmethod
+    def __convert_schema(_):
+        raise ArgumentError("不正な引数が与えられています．")
+
+    @__convert_schema.register(Dim1)
+    @staticmethod
+    def __convert_schema_dummy(schema: List):
+        raise ArgumentError("不正な引数が与えられています．")
+
+    @__convert_schema.register((Dim1, str))
+    @staticmethod
+    def __convert_schema_str(schema: List[str]) -> List[Schema]:
+        return [
+            Schema(name=name,
+                   type=ShareValueTypeEnum.SHARE_VALUE_TYPE_FIXED_POINT)
+            for name in schema]
+
+    @__convert_schema.register((Dim1, Schema))
+    @staticmethod
+    def __convert_schema_typed(
+            schema: List[Schema]) -> List[Schema]:
+        return schema
+
     @methoddispatch()
     def send_share(self, _):
-        raise ArgmentError("不正な引数が与えられています．")
+        raise ArgumentError("不正な引数が与えられています．")
 
     @send_share.register(Dim2)
     @send_share.register(Dim3)
-    def __send_share_impl(self, secrets: List, schema: List[str],
+    def __send_share_impl(self, secrets: List,
+                          schema: List[Union[str, Schema]],
                           matching_column: int,
                           piece_size: int) -> Dict:
         if piece_size < 1000 or piece_size > 1_000_000:
@@ -183,6 +222,8 @@ class QMPCServer:
                 "matching_column must be in the "
                 "range of 1 to the size of schema")
 
+        typed_schema: List[Schema] = QMPCServer.__convert_schema(schema)
+
         # TODO parse_csv経由でsend_shareをすると同じチェックをすることになる．
         if not format_check(secrets, schema):
             raise RuntimeError("規定されたフォーマットでないデータです．")
@@ -201,19 +242,20 @@ class QMPCServer:
         sent_at = str(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
 
         # リクエストパラメータを設定して非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(stub.SendShares,
-                                   SendSharesRequest(
-                                       data_id=data_id,
-                                       shares=json.dumps(s),
-                                       schema=schema,
-                                       piece_id=piece_id,
-                                       sent_at=sent_at,
-                                       matching_column=matching_column,
-                                       token=self.token))
-                   for piece_id, share_piece in enumerate(shares)
-                   for stub, s in zip(self.__client_stubs, share_piece)
-                   ]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry,
+                                       stub.SendShares,
+                                       SendSharesRequest(
+                                           data_id=data_id,
+                                           shares=json.dumps(s),
+                                           schema=typed_schema,
+                                           piece_id=piece_id,
+                                           sent_at=sent_at,
+                                           matching_column=matching_column,
+                                           token=self.token))
+                       for piece_id, share_piece in enumerate(shares)
+                       for stub, s in zip(self.__client_stubs, share_piece)
+                       ]
         is_ok, _ = QMPCServer.__futures_result(futures)
         return {"is_ok": is_ok, "data_id": data_id}
 
@@ -221,9 +263,9 @@ class QMPCServer:
         """ Shareを削除 """
         req = DeleteSharesRequest(dataIds=data_ids, token=self.token)
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(stub.DeleteShares, req)
-                   for stub in self.__client_stubs]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry, stub.DeleteShares, req)
+                       for stub in self.__client_stubs]
         is_ok, _ = QMPCServer.__futures_result(futures)
         return {"is_ok": is_ok}
 
@@ -231,7 +273,7 @@ class QMPCServer:
                             join_order: Tuple[List, List, List],
                             inp: Tuple[List, List]) -> Dict:
         if not self._argument_check(join_order):
-            raise ArgmentError("引数が正しくありません．")
+            raise ArgumentError("引数が正しくありません．")
 
         """ 計算リクエストを送信 """
         join_order_req = JoinOrder(
@@ -249,10 +291,12 @@ class QMPCServer:
         )
 
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        # JobidをMCから貰う関係で単一MC（現在はSP（ID=0）のみ対応）にリクエストを送る
-        futures = [executor.submit(
-            self.__client_stubs[0].ExecuteComputation, req)]
+        with ThreadPoolExecutor() as executor:
+            # JobidをMCから貰う関係で単一MC（現在はSP（ID=0）のみ対応）にリクエストを送る
+            futures = [
+                executor.submit(self.__retry,
+                                self.__client_stubs[0].ExecuteComputation,
+                                req)]
 
         is_ok, response = QMPCServer.__futures_result(futures)
         job_uuid = response[0].job_uuid if is_ok else None
@@ -261,10 +305,11 @@ class QMPCServer:
 
     def get_data_list(self) -> Dict:
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(stub.GetDataList,
-                                   GetDataListRequest(token=self.token))
-                   for stub in self.__client_stubs]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry,
+                                       stub.GetDataList,
+                                       GetDataListRequest(token=self.token))
+                       for stub in self.__client_stubs]
         is_ok, response = QMPCServer.__futures_result(futures)
         results = [eval(r.result) for r in response] if is_ok else None
 
@@ -277,10 +322,11 @@ class QMPCServer:
             token=self.token
         )
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(stub.GetElapsedTime,
-                                   req)
-                   for stub in self.__client_stubs]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry,
+                                       stub.GetElapsedTime,
+                                       req)
+                       for stub in self.__client_stubs]
         is_ok, response = QMPCServer.__futures_result(futures)
         elapsed_time = max([res.elapsed_time
                             for res in response]) if is_ok else None
@@ -295,11 +341,12 @@ class QMPCServer:
             token=self.token
         )
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(QMPCServer.__stream_result,
-                                   stub.GetComputationResult(req),
-                                   job_uuid, party, path)
-                   for party, stub in enumerate(self.__client_stubs)]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry,
+                                       QMPCServer.__stream_result,
+                                       stub.GetComputationResult(req),
+                                       job_uuid, party, path)
+                       for party, stub in enumerate(self.__client_stubs)]
         is_ok, response = QMPCServer.__futures_result(
             futures, enable_progress_bar=False)
         results_sorted = [sorted(res["responses"], key=lambda r: r.piece_id)
@@ -319,19 +366,23 @@ class QMPCServer:
             ]
 
         results: Optional[Any] = None
+        schema = None
+        is_table = False
         if not path and all_completed:
             for res in results_sorted:
-                is_table = False
                 is_dim2 = False
                 column_number = 0
                 result: Any = []
-                schema = []
+                schema_1p = []
                 for r in res:
                     if r.HasField("is_schema"):
                         if not is_table:
                             is_table = True
                         for val in r.result:
-                            schema.append(val)
+                            col_sch = google.protobuf.json_format.Parse(
+                                val, Schema())
+                            schema_1p.append(col_sch)
+                            # schema = col_sch
                     else:
                         if r.HasField("is_dim2"):
                             is_dim2 = True
@@ -343,13 +394,17 @@ class QMPCServer:
                 if is_dim2:
                     result = np.array(result).reshape(-1,
                                                       column_number).tolist()
-                result = {"schema": schema, "table": result} if is_table \
-                    else result
+                if len(schema_1p) != 0:
+                    schema = schema_1p
+                result = Share.convert_type(result, schema)
                 if results is None:
                     results = []
                 results.append(result)
 
         results = if_present(results, Share.recons)
+        results = if_present(results, Share.convert_type, schema)
+        if is_table:
+            results = {"schema": schema, "table": results}
         return {"is_ok": is_ok, "statuses": statuses,
                 "results": results, "progresses": progresses}
 
@@ -360,9 +415,9 @@ class QMPCServer:
             token=self.token
         )
         # 非同期にリクエスト送信
-        executor = ThreadPoolExecutor()
-        futures = [executor.submit(stub.GetJobErrorInfo, req)
-                   for stub in self.__client_stubs]
+        with ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.__retry, stub.GetJobErrorInfo, req)
+                       for stub in self.__client_stubs]
         is_ok, response = QMPCServer.__futures_result(
             futures, enable_progress_bar=False)
 
