@@ -2,11 +2,12 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from decimal import Decimal
 from typing import ClassVar, List
+import math
 
 from nacl.utils import random, randombytes_deterministic
 
+from ..exception import ArgumentError
 from .overload_tools import methoddispatch
-from ..exception import ArgmentError
 
 
 # 乱数生成のインタフェース
@@ -29,17 +30,18 @@ class ChaCha20(RandomInterface):
 
     @methoddispatch()
     def get(self, a, b):
-        raise ArgmentError(
+        raise ArgumentError(
             "乱数の閾値はどちらもintもしくはdecimalでなければなりません．"
             f"a is {type(a)}, b is {type(b)}")
 
     @get.register(int)
     def __get_int(self, a: int, b: int) -> int:
-        # TRNGで32byte(256bit)生成
+        # TRNGで [a,b) の乱数生成
         self.__exception_check(a, b)
-        byte_val: bytes = self.__get_32byte()
-        int32byte = int.from_bytes(byte_val, "big")
-        return int32byte % (b-a)+a
+        interval_byte = self.__get_byte_size(b-a)
+        byte_val: bytes = random(interval_byte)
+        int_val = int.from_bytes(byte_val, "big")
+        return int_val % (b - a) + a
 
     @get.register(Decimal)
     def __get_decimal(self, a: Decimal, b: Decimal) -> Decimal:
@@ -50,14 +52,14 @@ class ChaCha20(RandomInterface):
 
     @methoddispatch()
     def get_list(self, a, b, size: int):
-        raise ArgmentError(
+        raise ArgumentError(
             "乱数の閾値はどちらもintもしくはdecimalでなければなりません．"
             f"a is {type(a)}, b is {type(b)}")
 
     @get_list.register(int)
     def __get_list_int(self, a: int, b: int, size: int) -> List[int]:
         # TRNGの32byteをseedとしてCSPRNGでsize分生成
-        byte_size: int = 16
+        byte_size: int = self.__get_byte_size(b-a)
         self.__exception_check(a, b)
         seed: bytes = self.__get_32byte()
         bytes_list: bytes = randombytes_deterministic(size*byte_size, seed)
@@ -74,15 +76,19 @@ class ChaCha20(RandomInterface):
         return [Decimal(val-self.mn)/(self.mx-self.mn)*(b-a)+a
                 for val in valList]
 
+    def __get_byte_size(self, x: int) -> int:
+        # 整数の byte サイズを取得
+        return max(math.ceil(math.log2(x))//8 + 1, 32)
+
     def __get_32byte(self) -> bytes:
         return random()
 
     def __exception_check(self, a, b) -> None:
         if a >= b:
-            raise ArgmentError(
+            raise ArgumentError(
                 "乱数の下限は上限より小さい必要があります．"
                 f"{a} < {b}")
         if type(a) != type(b):
-            raise ArgmentError(
+            raise ArgumentError(
                 "乱数の下限と上限の型は一致させる必要があります．"
                 f"{type(a)} != {type(b)}")
