@@ -1,11 +1,18 @@
 import math
+from decimal import Decimal
 from typing import List
 
 import numpy as np
 import pytest
+
+from quickmpc.exception import ArgumentError
 from quickmpc.share import Share
 
 
+def sharize_params(secrets=[1], party_size=3):
+    return (secrets, party_size)
+
+
 class TestQMPC:
 
     @pytest.mark.parametrize(
@@ -25,10 +32,11 @@ class TestQMPC:
     def test_sharize(self, secrets: list):
         """ nパーティのシェア化が正しくできているかのTest """
         for party_size in range(2, 10):
-            shares: np.ndarray = np.vectorize(float)(
+            shares: np.ndarray = np.vectorize(Decimal)(
                 Share.sharize(secrets, party_size=party_size))
             assert (len(shares) == party_size)
-            assert (np.allclose(secrets, np.sum(shares, axis=0)))
+            assert (np.allclose(secrets,
+                                np.vectorize(float)(np.sum(shares, axis=0))))
 
     @pytest.mark.parametrize(
         ("secrets"),
@@ -37,10 +45,10 @@ class TestQMPC:
     def test_sharize_scalar(self, secrets: list):
         """ nパーティのスカラ値のシェア化が正しくできているかのTest """
         for party_size in range(2, 10):
-            shares: np.ndarray = np.vectorize(float)(
+            shares: np.ndarray = np.vectorize(Decimal)(
                 Share.sharize(secrets, party_size=party_size))
             assert (len(shares) == party_size)
-            assert (np.sum(shares) == pytest.approx(secrets))
+            assert (float(np.sum(shares)) == pytest.approx(secrets))
 
     @pytest.mark.parametrize(
         ("secrets"),
@@ -72,20 +80,77 @@ class TestQMPC:
             for share in share_json:
                 for key, val in dfs(share).items():
                     if key not in share_map:
-                        share_map[key] = 0
-                    share_map[key] += float(val)
+                        share_map[key] = Decimal(0)
+                    share_map[key] += Decimal(val)
             for key, _ in dfs(share).items():
-                assert (math.isclose(share_map[key], true_map[key]))
+                assert (np.isclose(float(share_map[key]), true_map[key]))
 
-    def test_sharize_errorhandring(self):
+    @pytest.mark.parametrize(
+        ("secrets"),
+        [
+            # 1次元配列の秘密情報
+            ([1e999]),
+            # 2次元配列の秘密情報
+            ([[1e999]])
+        ]
+    )
+    def test_sharize_convert_to_inf(self, secrets: list):
+        """ Infinityがinfに変換されているかのTest """
+        for party_size in range(2, 10):
+            shares = Share.sharize(secrets, party_size=party_size)
+
+            def is_inf(val):
+                ok: bool = True
+                if isinstance(val, list):
+                    for v in val:
+                        ok &= is_inf(v)
+                else:
+                    ok &= (val == 'inf')
+                return ok
+
+            assert (is_inf(shares[0]))
+
+    @pytest.mark.parametrize(
+        ("secrets"),
+        [
+            # zero
+            (0),
+            # int_max
+            (1e1200),
+            # int_min
+            (-1e1200),
+            # float_min_plus
+            (1e-50),
+            # float_min_minus
+            (-1e-50),
+        ]
+    )
+    def test_sharize_edge(self, secrets: float):
+        """ nパーティのシェア化が正しくできているかのTest """
+        for party_size in range(2, 10):
+            shares: np.ndarray = np.vectorize(Decimal)(
+                Share.sharize(secrets, party_size=party_size))
+            assert (len(shares) == party_size)
+            assert math.isclose(secrets, sum(shares), abs_tol=1e-5)
+
+    @pytest.mark.parametrize(
+        ("args", "expected_exception"),
+        [
+            # string
+            (sharize_params(secrets="str"), ArgumentError),
+            # array string
+            (sharize_params(secrets="[str]"), ArgumentError),
+            # 列が異なる
+            (sharize_params(secrets=[[1, 2], [1]]), ValueError),
+            # 3-dim
+            (sharize_params(secrets=[[[1]]]), ArgumentError),
+            # empty
+            (sharize_params(secrets=[[], []]), ValueError),
+            # party_sizeが1
+            (sharize_params(party_size=1), ArgumentError),
+        ]
+    )
+    def test_sharize_errorhandring(self, args, expected_exception):
         """ 異常値を与えてエラーが出るかTest """
-        with pytest.raises(Exception):
-            Share.sharize("hey")
-        with pytest.raises(Exception):
-            Share.sharize(["str"])
-        with pytest.raises(Exception):
-            Share.sharize([[1, 2], [1]])
-        with pytest.raises(Exception):
-            Share.sharize([[[1]]])
-        with pytest.raises(Exception):
-            Share.sharize([[], []])
+        with pytest.raises(expected_exception):
+            Share.sharize(*args)
